########################################################################################################
# This is a complete class, you don't have to do anything except understand it and use it for
# the DBMserver class
# Use the small files for development and testing:
#                      table1.csv and table2.csv
# that are supplied in this directory.
# The final project will be tested on the full Database system in the csv file: db.csv
# This is however a very large file, so is not recommended for development and debugging
# It can be always be downloaded from:
#            tinyurl.com/samyz/python/projects/db.zip
# You need to unzip it to get: db.csv
#
# A databse record will be modeled as Python dictionary:
#   For example The CSV line:
#
#     female, Mrs., Margaret, Bremer, 1663 Cody Ridge Road, Guymon, OK, 73942, United States, MargaretGBremer@cuvox.de, 580-528-0467, Wade, 7/11/1955, 446-30-1063, 1Z 85A 964 51 2250 596 5, Real estate closer, Gov Mart, 2005 MG ZR, TuitionDepartment.com, B+
#
#   Will become a Python dictionary object:
#
#       record['Gender']          = 'female'
#       record['Title']           = 'Mrs.'
#       record['GivenName']       = 'Margaret'
#       record['Surname']         = 'Bremer'
#       record['StreetAddress']   = '1663 Cody Ridge Road'
#       record['City']            = 'Guymon'
#       record['State']           = 'OK'
#       record['ZipCode']         = '73942'
#       record['CountryFull']     = 'United States'
#       record['EmailAddress']    = 'MargaretGBremer@cuvox.de'
#       record['TelephoneNumber'] = '580-528-0467'
#       record['MothersMaiden']   = 'Wade'
#       record['Birthday']        = '7/11/1955'
#       record['NationalID']      = '446-30-1063'
#       record['UPS']             = '1Z 85A 964 51 2250 596 5'
#       record['Occupation']      = 'Real estate closer'
#       record['Company']         = 'Gov Mart'
#       record['Vehicle']         = '2005 MG ZR'
#       record['Domain']          = 'TuitionDepartment.com'
#       record['BloodType']       = 'B+'
#       ...
#
# A DataBase System will be modeled as a Python class that reads all the records and provides
# several services to access and manipulate these records.
#
#    CLASS:
#        DataBaseManager
#
#    CONSTRUCTOR:
#        To create a DataBaseManager object we only need a CSV file which we parse
#        and create all records. We keep the records in memory for any future query.
#        This way we save multiple parsings of the same file for different queries.
#        Python reserves the __init__ function to define the constructor.
#        This function is automatically invoked when we create an instance of the class.
#
#    METHODS:
#        select(filter)                 - Select records that satisfy a given filter
#        write(keys, filter, outfile)   - Write a csv file for table from a given filter and keys
#        record(n)                      - Return record number n
#
#    MEMBERS:
#        keys                           - Column keys
#        size                           - Data base size (number of records)
#
# INTERFACE PLAN: HOW DO WE WANT TO USE OUR CLASS?
# -------------------------------------------------
# At this stage we write down many examples of how we intend to use our class?
# We are allowed to express all our dreams and fantasies on what our class can do and
# how easy it's doing it.
# Later, when we implement the class, we will look at these examples to see
# what class methods and class members we need to design?
# Here is a good plan:
#     dbm = DataBaseManager("C:/Projects/db.csv")
#
#     dbm.size
#       => 299769
#
#     dbm.keys
#       => ['Gender', 'Title', 'GivenName', 'Surname', 'StreetAddress', ...]
#
#     dbm.record(17)
#       => {'Birthday': '8/28/1949', 'BloodType': 'AB+', 'Centimeters': '171', 'City': 'Atlanta', ... }
#
#     dbm.file
#       => C:/Projects/db.csv
#
#     dbm.save("c:/Projects/db2.csv")
#
# To get all people from Florida with blood type B+:
#
#     def filter(record):
#         State = record['State']
#         blood = record['BloodType']
#         if state == 'FL' and blood == 'B+':
#             return record
#         else:
#             return None
#
#     records = dbm.select(filter)
#
# To write a table of all these people with the following header:
#
#     KEYS = ['GivenName', 'Surname', 'NationalID', 'State', 'BloodType', 'EmailAddress']
#
# We will use the following code:
#
#     dbm.write(KEYS, filter, "C:/Projects/table5.csv")
#
# In general, the class design and implementation are driven by clients requirements.
# In our case, the requirements are specified in problems 4-10 of project 2.
# So class usage will become clearer in the solutions of Project 2 (see later).
#
##########################################################################################################

class DataBaseManager:
    def __init__(self, file):
        self.file = file
        f = open(file, 'r')
        self.keys = f.next().strip().split(",")
        self.__records = []
        for line in f:
            record = {}
            field_iterator = iter(line.strip().split(","))
            for key in self.keys:
                record[key] = field_iterator.next()
            self.__records.append(record)
        f.close()
        self.size = len(self.__records)

    def select(self, filter):
        table = []
        for rec in self.__records:
            record = filter(rec)
            if record is not None:
                table.append(record)
        return table

    def record(self,n):
        return self.__records[n]


