from socket_utils import *

class DBMserver:
    def __init__(self, dbfile, port):
        # server initialization
        # server should be ready and start working in this method
        self.sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
        self.sock.bind(("",port))     # bind to all local host interfaces
        self.sock.listen(25)          # set maximum accept rate to 25 connections
        self.port = port
        self.dbm = DataBaseManager(dbfile)
        self.accept_clients()

    def close(self):
        self.sock.close()

    def accept_clients(self):
        # This is the server loop for accepting clients
        print "Starting DBMserver on localhost, port:", self.port
        while True:
            client_socket, client_address = self.sock.accept()
            date = time.ctime()
            host, port = client_address
            print "Connected to client host=%s, port=%d, DATE: %s" % (host, port, date)
            thread = ResponseThread(client_socket, self.dbm)   # must pass dbm to ResponseThread so he can take care of client requests
            thread.start()
            print "Launched Response Thread =", thread.getName()

#----------------------- end of DBMserver class ----------------------------

class ResponseThread(Thread):
    def __init__(self, client_socket, dbm):
        Thread.__init__(self)
        self.sock = client_socket
        self.dbm = dbm

    def run(self):
        while True:
            message = recv_size(self.sock)
            # Client loop:
            # Get client requests: SIZE, KEYS, FILTERS, QUERY, ...... (read the specification)
            # and dispatch them to one of:
            # do_KEYS, do_SIZE, do_RECORD, do_QUERY, .....

    def do_KEYS(self):
        # Send database keys to the client

    def do_SIZE(self):
        # Send database size to the client

    def do_RECORD(self):
        # send record (number) to client
        rec_number = self.args[1]
        # continue here ......

    def do_FILTERS(self):
        # receive filters file from client
        # if you get it, send good status, else complain ...

    def do_QUERY(self):
        # Process client query command

    def do_QUIT(self):
        self.send("205 QUIT request OK: closing connection. BYE.")
        # hand QUIT request: close connection, ...
        # nice to have: print "Closed connection to client %s:%s, DATE: %s" % (host, port, date)

    # Our private send method is a simple wrapper around the send_size function
    # this is useful in case we want to change the implementation in the future
    # use the following method everytime you need to send a message to the server
    def send(self, message):
        send_size(self.sock, message)


